#pragma once
#include "GuiComponent.hpp"
#include <raylibs/raygui.h>
#include <string>


class GuiMovableWindow : public sva::GuiComponent
{
    private:
        bool m_DragWindow = false;
        Vector2 m_PanOffset = { 0,0 };
    protected:
        Rectangle m_WndRect = { 20,20, 200, 100 };
        bool m_WindowOpen = true;
    public:

        virtual int input() override
        {
            if (!m_WindowOpen) return 0;
            Vector2 mouse_pos = GetMousePosition();

            if (IsMouseButtonPressed(MOUSE_BUTTON_LEFT) && !m_DragWindow)
            {
                if (CheckCollisionPointRec(mouse_pos, { m_WndRect.x, m_WndRect.y, m_WndRect.width - 24, 20 }))
                {
                    m_DragWindow = true;
                    m_PanOffset = { mouse_pos.x - m_WndRect.x, mouse_pos.y - m_WndRect.y };
                }
            }

            if (m_DragWindow)
            {
                m_WndRect.x = (mouse_pos.x - m_PanOffset.x);
                m_WndRect.y = (mouse_pos.y - m_PanOffset.y);
                if (IsMouseButtonReleased(MOUSE_BUTTON_LEFT)) m_DragWindow = false;
            }
            return rinput(mouse_pos);
        }

        virtual int rinput(Vector2& mouse_position) { return 0; };

        virtual int draw() override
        {
            if (!m_WindowOpen) return 0;

            m_WindowOpen = !GuiWindowBox(m_WndRect, "Example Movable Window");

            return 0;
        }
};


/****************************************************
 * A "Are you sure you want to exit?" Window.
 *
 * Yes is default
 * TODO: Make the Yes-Button standout as the default
****************************************************/
class SafeClosePopup final : public GuiMovableWindow
{
    Vector2 anchor03 = { 0,0 };

    bool m_CQB_YesButton = false;
    bool m_CQB_NoButton = false;
    
    public:

    ~SafeClosePopup() {
        CloseWindow();
    }

    void onAttach() override
    {
        m_WindowOpen = false;
    }

    int draw() override
    {
        if (WindowShouldClose())
        {
            if (m_WindowOpen) {
                CloseWindow();
            }
            else {
                OpenWindow();
            }
            onResize();
        }

        if (m_WindowOpen)
        {
            GuiUnlock();
            anchor03 = { m_WndRect.x + 168, m_WndRect.y + 88 };

            m_WindowOpen = !GuiWindowBox(m_WndRect, "#191# Are you sure you want to close this program?");
            m_CQB_YesButton = GuiButton({ anchor03.x + -152, anchor03.y + 32, 120, 24 }, "#112#Yes");
            m_CQB_NoButton = GuiButton({ anchor03.x + 24, anchor03.y + 32, 120, 24 }, "#113#No");
            GuiLabel({ anchor03.x + -104, anchor03.y + -40, 208, 24 }, "Are you sure you want to close this?");
            GuiLabel({ anchor03.x + -56, anchor03.y + -8, 120, 24 }, "Press \"Yes\" to close");
            if (m_CQB_YesButton)
            {
                *m_WndRunning = false;
                return 1;
            }
            if (m_CQB_NoButton) CloseWindow(); 
        }
        return 0;
    }

    int rinput(Vector2& mouse_position) override
    {
        if (IsKeyReleased(KEY_ENTER)) {
            *m_WndRunning = false;
            return 1;
        }
        GuiLock();
        return 0;
    }


    void onResize() override
    {
        anchor03 = { static_cast<float>(m_WndWidth) / 2 , static_cast<float>(m_WndHeight) / 2 };
        m_WndRect = { anchor03.x + -168, anchor03.y + -88, 328, 160 };
    }

    public:
    void OpenWindow() {
        GuiLock();
        m_WindowOpen = true;
    }

    void CloseWindow() {
        m_WindowOpen = false;
        GuiUnlock();
    }
};


class SettingsComponent final : public sva::GuiComponent {
    public:
        struct {
            bool borderlessFullscreen;
            uint32_t state;
        } values;

        void onAttach() override {
            values.borderlessFullscreen = false;
            values.state = 0;
        }

        int draw() override {
            if(GuiButton({100, 100, 300, 30}, "This should not work when the safe window is open!")) {
                std::cout << "got pressed\n";
            }
            return 0;
        }
};

